from ichec_django_core.models import Member, Organization, Address

from ichec_django_core.client.resources import create_organizations
from ichec_django_core.test import (
    setup_default_users_and_groups,
    setup_default_organizations,
    add_group_permissions,
    AuthAPITestCase,
)


class TestOrganizationView(AuthAPITestCase):

    template = create_organizations(1)[0]

    def setUp(self):
        self.url = "/api/organizations/"

        setup_default_users_and_groups()

        add_group_permissions(
            "admins",
            Organization,
            ["change_organization", "add_organization", "delete_organization"],
        )

        add_group_permissions(
            "members",
            Organization,
            ["view_organization"],
        )

        orgs = setup_default_organizations(Member.objects.get(username="other_user"))
        self.internal_org, self.public_org, self.inactive_org = orgs

    def test_list_access(self):

        scenarios = (
            ["", 1, 200, "Public can list public items"],
            ["regular_user", 1, 200, "Registered can list public items"],
            ["member", 2, 200, "Member can list all items"],
            ["other_user", 2, 200, "Item member can list item"],
        )
        for user, count, status, msg in scenarios:
            if user:
                response = self.assert_status(self.auth_list(user), status, msg)
            else:
                response = self.assert_status(self.public_list(), status, msg)

            self.assertEqual(response.body.count, count, msg)

    def test_detail_access(self):

        public = self.public_org.id
        internal = self.internal_org.id
        scenarios = (
            ["", public, "public", 200, "Public can view public item"],
            ["", internal, "public", 404, "Public can't view internal item"],
            ["regular_user", public, "public", 200, "Registered can view public item"],
            [
                "regular_user",
                internal,
                "public",
                404,
                "Registered can't view internal item",
            ],
            ["member", public, "", 200, "Member can view public item"],
            ["member", internal, "", 200, "Item Member can view internal item"],
            ["other_user", internal, "", 200, "Item member can view item"],
        )

        for user, item, role, status, msg in scenarios:
            if user:
                self.assert_status(self.auth_detail(user, item, role), status, msg)
            else:
                self.assert_status(self.detail(item, role), status, msg)

    def test_create_permissions(self):

        scenarios = (
            ["", 401, "Public can't create, no auth"],
            ["regular_user", 403, "Registered can't create, no perm"],
            ["admin_user", 201, "Admin can create"],
        )

        for user, status, msg in scenarios:
            if user:
                self.assert_status(self.auth_create(user, self.template), status, msg)
            else:
                self.assert_status(self.create(self.template), status, msg)

    def _create_for_update(
        self, creator, members: list | None = None, member_key: str = "members"
    ):

        if members:
            template = self.template.model_copy(
                update={"members": [self.get_member_url(m) for m in members]}
            )
        else:
            template = self.template

        created = self.assert_201(self.auth_create(creator, template)).body
        return created.model_copy(
            update={"address": created.address.model_copy(update={"country": "FR"})}
        )

    def _verify_update(self, updated):
        self.assertEqual(updated.body.address.country, "FR")

    def test_update_permissions(self):

        scenarios = (
            ["", 401, False, "Public can't update, no auth"],
            ["regular_user", 404, False, "Registered can't update, no list"],
            ["other_user", 200, True, "Item member can update"],
            ["admin_user", 200, False, "Admin can update"],
        )

        for user, status, member, msg in scenarios:
            print(msg)
            if member:
                created = self._create_for_update("admin_user", [user])
            else:
                created = self._create_for_update("admin_user")

            print(created)

            updated = None
            if user:
                updated = self.assert_status(
                    self.auth_update(user, created.id, created), status, msg
                )
            else:
                self.assert_status(self.update(created.id, created), status, msg)

            if updated:
                self._verify_update(updated)

    def test_delete_permissions(self):

        scenarios = (
            ["", 401, False, "Public can't delete, no auth"],
            ["regular_user", 403, False, "Registered can't delete"],
            ["other_user", 403, True, "Item member can't delete"],
            ["admin_user", 204, False, "Admin can delete"],
        )

        for user, status, member, msg in scenarios:
            if member:
                created = self._create_for_update("admin_user", [user])
            else:
                created = self._create_for_update("admin_user")

            if user:
                self.assert_status(self.auth_delete(user, created.id), status, msg)
            else:
                self.assert_status(self.delete(created.id), status, msg)

    def test_filters(self):
        pass

    def test_fields(self):
        pass



import unicodedata
import logging

from django.contrib.auth.models import User
from django.core.exceptions import SuspiciousOperation
from django.urls import reverse
from django.core.mail import EmailMultiAlternatives
from django.template.loader import render_to_string
from django.conf import settings

from mozilla_django_oidc.auth import OIDCAuthenticationBackend
from mozilla_django_oidc.utils import absolutify, import_from_settings

from ..models.member import Member

logger = logging.getLogger(__name__)


def generate_username(email):

    return unicodedata.normalize("NFKC", email.split("@")[0][:150])


def store_tokens(session, access_token, id_token, refresh_token):
    if import_from_settings("OIDC_STORE_ACCESS_TOKEN", False):
        session["oidc_access_token"] = access_token

    if import_from_settings("OIDC_STORE_ID_TOKEN", False):
        session["oidc_id_token"] = id_token

    if import_from_settings("OIDC_STORE_REFRESH_TOKEN", True):
        session["oidc_refresh_token"] = refresh_token


class OIDCAuthenticationBackendWithRefresh(OIDCAuthenticationBackend):
    """
    Modified version based on https://github.com/mozilla/mozilla-django-oidc/pull/377
    to add refresh token storage as part of the authentication process.

    They can be used later in the session refresh middleware

    """

    def authenticate(self, request, **kwargs):
        """Authenticates a user based on the OIDC code flow."""

        self.request = request
        if not self.request:
            return None

        state = self.request.GET.get("state")
        code = self.request.GET.get("code")
        nonce = kwargs.pop("nonce", None)
        code_verifier = kwargs.pop("code_verifier", None)

        if not code or not state:
            return None

        reverse_url = self.get_settings(
            "OIDC_AUTHENTICATION_CALLBACK_URL", "oidc_authentication_callback"
        )

        token_payload = {
            "client_id": self.OIDC_RP_CLIENT_ID,
            "client_secret": self.OIDC_RP_CLIENT_SECRET,
            "grant_type": "authorization_code",
            "code": code,
            "redirect_uri": absolutify(self.request, reverse(reverse_url)),
        }

        if code_verifier is not None:
            token_payload.update({"code_verifier": code_verifier})

        # Get the token
        token_info = self.get_token(token_payload)
        id_token = token_info.get("id_token")
        access_token = token_info.get("access_token")
        refresh_token = token_info.get("refresh_token")

        # Validate the token
        payload = self.verify_token(id_token, nonce=nonce)
        if payload:
            session = self.request.session
            store_tokens(session, access_token, id_token, refresh_token)
            try:
                return self.get_or_create_user(access_token, id_token, payload)
            except SuspiciousOperation as exc:
                logger.warning("Failed to get or create user: %s", exc)
                return None

        return None


class MemberOIDCAuthenticationBackend(OIDCAuthenticationBackendWithRefresh):

    def __init__(self, *args, **kwargs):
        self.default_scopes = ("openid", "email", "profile")
        super().__init__(self, args, kwargs)

    def get_scopes(self):
        env_settings = self.get_settings("OIDC_RP_SCOPES", None)
        if env_settings:
            return env_settings.split()
        return self.default_scopes

    def verify_claims(self, claims):
        if "email" not in claims:
            return False
        return True

    def get_username(self, claims):
        if "preferred_username" in claims:
            logger.info("Using preferred user name")
            return claims["preferred_username"]
        return generate_username(claims.get("email"))

    def create_user(self, claims):
        logger.info("Creating new user")

        user = super().create_user(claims)

        logger.info("Updating member based on OIDC claims")
        member = Member.objects.get(id=user.id)

        # Assuming email verification etc has happened on IDP
        member.verified = True
        member.first_name = claims.get("given_name", "")
        member.last_name = claims.get("family_name", "")
        member.save()

        if settings.EMAIL_ON_SELF_REGISTER:
            self.email_new_user(user)
        return user

    def email_new_user(self, user):

        context = {
            "email": user.email,
        }

        emails_dir = "ichec_django_core/emails"

        template = emails_dir + "/admin_user_not_invited"
        text = render_to_string(template + ".txt", context=context)
        html = render_to_string(template + ".html", context=context)

        superusers_emails = User.objects.filter(is_superuser=True).values_list("email")
        if superusers_emails:
            email = superusers_emails[0]
        else:
            return

        msg = EmailMultiAlternatives(
            subject="New Self-Registered User",
            body=text,
            from_email=settings.DEFAULT_EMAIL_SENDER,
            to=[email],
            headers={"List-Unsubscribe": f"<mailto:{settings.DEFAULT_EMAIL_UNSUB}"},
        )

        msg.attach_alternative(html, "text/html")
        msg.send()

    def email_invited_user(self, user):
        context = {
            "email": user.email,
        }

        emails_dir = "ichec_django_core/emails"

        template = emails_dir + "/admin_user_invited"
        text = render_to_string(template + ".txt", context=context)
        html = render_to_string(template + ".html", context=context)

        superusers_emails = User.objects.filter(is_superuser=True).values_list("email")
        if superusers_emails:
            email = superusers_emails[0]
        else:
            return

        msg = EmailMultiAlternatives(
            subject="New Invited User",
            body=text,
            from_email=settings.DEFAULT_EMAIL_SENDER,
            to=[email],
            headers={"List-Unsubscribe": f"<mailto:{settings.DEFAULT_EMAIL_UNSUB}"},
        )

        msg.attach_alternative(html, "text/html")
        msg.send()

    def update_user(self, user, claims):
        member = Member.objects.get(id=user.id)

        # Assuming email verification etc has now happened on IDP
        if not member.verified:
            member.verified = True
            member.first_name = claims.get("given_name", "")
            member.last_name = claims.get("family_name", "")
            member.save()

            if settings.EMAIL_ON_SELF_REGISTER:
                self.email_invited_user(user)

        return user



from django.contrib.auth.models import Group

from rest_framework import viewsets

from ichec_django_core.serializers import GroupDetailSerializer, GroupListSerializer

from .core import SearchableModelViewSet
from .permissions import CustomDjangoModelPermissions, get_permission


class GroupPermissions(CustomDjangoModelPermissions):

    intercept_methods = ["GET"]
    allow_django_fallback = False

    def object_perm_check(self, obj, request) -> bool:
        return request.user.has_perm(get_permission(Group, "view"))


class GroupViewSet(SearchableModelViewSet):
    model = Group
    queryset = Group.objects.all()
    serializer_class = GroupListSerializer
    permission_classes = [GroupPermissions]

    ordering_fields: tuple[str, ...] = ("name",)
    ordering: tuple[str, ...] = ("name",)
    search_fields: tuple[str, ...] = ("name",)

    serializers = {
        "retrieve": GroupDetailSerializer,
        "list": GroupListSerializer,
        "create": GroupDetailSerializer,
        "update": GroupDetailSerializer,
        "partial_update": GroupDetailSerializer,
    }

    def get_queryset(self):
        """
        Public users can only see public Orgs
        Registered users can see pubic Orgs or orgs they are members of
        Users with view permission can see all Orgs
        """

        queryset = viewsets.ModelViewSet.get_queryset(self)

        if not self.has_model_view_permission():
            queryset = queryset.filter(user__id=self.request.user.id)

        user_id = self.request.query_params.get("user")
        if user_id is not None:
            queryset = queryset.filter(user__id=user_id)
        return queryset



from datetime import datetime

from pydantic import BaseModel, Extra


class File:

    def get_content(self):
        pass


class Resource(BaseModel, frozen=True):

    id: int


_TYPE_TO_PATH: dict = {}

_PATH_TO_TYPES: dict = {}


class FileWrapper(BaseModel, frozen=True):
    content: bytes
    filename: str


class ModelWithFiles(BaseModel, frozen=True):
    model: BaseModel
    files: dict[str, FileWrapper] = {}


def register_types(label: str, types: dict):

    if "create" not in types:
        types["create"] = types["detail"]
    if "list" not in types:
        types["list"] = types["detail"]

    _PATH_TO_TYPES[label] = types

    for t in types.values():
        _TYPE_TO_PATH[t.__name__] = label


def get_path_from_item(resource: BaseModel) -> str:
    return _TYPE_TO_PATH[type(resource).__name__]


def get_path_from_type(item_t) -> str:
    return _TYPE_TO_PATH[item_t.__name__]


def get_type(path: str, action: str):
    return _PATH_TO_TYPES[path][action]


class Identifiable(Resource, frozen=True):
    url: str


class Timestamped(Identifiable, frozen=True):
    created_at: datetime
    updated_at: datetime


class MemberIdentifierBase(BaseModel, frozen=True):
    id_type: str
    value: str


class MemberIdentifierCreate(MemberIdentifierBase, frozen=True):
    pass


class MemberIdentifierDetail(Resource, MemberIdentifierBase, frozen=True):
    pass


class MemberBase(BaseModel, frozen=True):
    username: str
    email: str
    first_name: str
    last_name: str | None = None
    phone: str | None = None


class MemberCreate(MemberBase, frozen=True):
    profile: str | None = None
    identifiers: list[MemberIdentifierCreate] = []

    class Config:
        extra = Extra.forbid


class MemberList(Timestamped, MemberBase, frozen=True):
    organizations: list[str] = []
    profile: str | None = None
    profile_thumbnail: str | None = None
    identifiers: list[MemberIdentifierDetail] = []
    verified: bool

    class Config:
        extra = Extra.forbid


class MemberPreferencesBase(BaseModel):
    notifications: dict[str, str]


class MemberPreferencesCreate(BaseModel):
    pass


class MemberPreferencesDetail(MemberPreferencesBase):
    id: int

    class Config:
        extra = Extra.forbid


class MemberDetail(MemberList, frozen=True):
    permissions: list[str] = []
    preferences: MemberPreferencesDetail | None = None

    class Config:
        extra = Extra.forbid


class GroupBase(BaseModel, frozen=True):
    name: str


class GroupList(Identifiable, GroupBase, frozen=True):

    class Config:
        extra = Extra.forbid


class GroupCreate(GroupBase, frozen=True):
    class Config:
        extra = Extra.forbid


class GroupDetail(Identifiable, GroupBase, frozen=True):
    user_set: list[str]

    class Config:
        extra = Extra.forbid


class NotificationBase(BaseModel, frozen=True):
    message: str
    link: str
    read_time: datetime | None


class NotificationDetail(Timestamped, NotificationBase, frozen=True):
    user: str

    class Config:
        extra = Extra.forbid


class FeedbackBase(BaseModel, frozen=True):
    comments: str


class FeedbackCreate(FeedbackBase, frozen=True):
    class Config:
        extra = Extra.forbid


class FeedbackDetail(Timestamped, FeedbackBase, frozen=True):
    creator: str

    class Config:
        extra = Extra.forbid


class AddressBase(BaseModel, frozen=True):
    line1: str
    line2: str | None = None
    line3: str | None = None
    city: str | None = None
    region: str
    postcode: str | None = None
    country: str


class AddressCreate(AddressBase, frozen=True):
    class Config:
        extra = Extra.forbid


class AddressDetail(AddressBase, frozen=True):
    id: int
    country_name: str
    country_flag: str

    class Config:
        extra = Extra.forbid


class OrganizationBase(BaseModel, frozen=True):
    name: str
    acronym: str | None = None
    description: str | None = None
    website: str | None = None


class OrganizationCreate(OrganizationBase, frozen=True):
    address: AddressCreate
    members: list[str] = []

    class Config:
        extra = Extra.forbid


class OrganizationPublicDetail(Timestamped, OrganizationBase, frozen=True):
    address: AddressDetail

    class Config:
        extra = Extra.forbid


class OrganizationDetail(OrganizationPublicDetail, frozen=True):
    public: bool
    members: list[str] = []

    class Config:
        extra = Extra.forbid


class FormFieldBase(BaseModel, frozen=True):

    label: str
    key: str
    required: bool = False
    description: str | None = ""
    tooltip: str | None = ""
    options: str | None = None
    default: str | None = None
    field_type: str
    order: int = 0


class FormFieldCreate(FormFieldBase, frozen=True):
    template: str | None = None

    class Config:
        extra = Extra.forbid


class FormFieldDetail(Resource, FormFieldBase, frozen=True):
    template: bool | None

    class Config:
        extra = Extra.forbid


class FormGroupBase(BaseModel, frozen=True):
    label: str | None = None
    description: str | None = None
    order: int = 0


class FormGroupCreate(FormGroupBase, frozen=True):
    fields: list[FormFieldCreate] = []

    class Config:
        extra = Extra.forbid


class FormGroupDetail(Resource, FormGroupBase, frozen=True):
    fields: list[FormFieldDetail] = []


class FormBase(BaseModel, frozen=True):
    pass


class FormCreate(FormBase, frozen=True):
    groups: list[FormGroupCreate] = []


class FormDetail(Resource, FormBase, frozen=True):
    groups: list[FormGroupDetail] = []


class FormFieldValueBase(BaseModel, frozen=True):
    value: str
    asset: str | None = None


class FormFieldValueCreate(FormFieldValueBase, frozen=True):
    field: int


class FormFieldValueUpdate(Resource, FormFieldValueBase, frozen=True):
    field: int


class FormFieldValueDetail(Resource, FormFieldValueBase, frozen=True):
    field: FormFieldDetail

    def to_update(self):
        return FormFieldValueUpdate(
            id=self.id, value=self.value, asset=self.asset, field=self.field.id
        )


class PopulatedFormBase(BaseModel, frozen=True):
    pass


class PopulatedFormCreate(PopulatedFormBase, frozen=True):
    values: list[FormFieldValueCreate] = []


class PopulatedFormUpdate(PopulatedFormBase, frozen=True):
    values: list[FormFieldValueUpdate] = []


class PopulatedFormDetail(PopulatedFormBase, frozen=True):
    values: list[FormFieldValueDetail] = []

    def to_update(self):
        return PopulatedFormUpdate(values=[v.to_update() for v in self.values])


register_types(
    "members", {"create": MemberCreate, "detail": MemberDetail, "list": MemberList}
)
register_types(
    "groups", {"detail": GroupDetail, "create": GroupCreate, "list": GroupList}
)
register_types("notifications", {"detail": NotificationDetail})
register_types("feedback", {"detail": FeedbackDetail, "create": FeedbackBase})
register_types(
    "organizations",
    {
        "create": OrganizationCreate,
        "detail": OrganizationDetail,
        "detail-public": OrganizationPublicDetail,
        "list": OrganizationPublicDetail,
    },
)



from pathlib import Path

from pydantic import BaseModel
from django.core import mail

from ichec_django_core.models import Member
from ichec_django_core.client import MemberIdentifierCreate, MemberDetail
from ichec_django_core.client.resources import create_members

from ichec_django_core.test import (
    AuthAPITestCase,
    setup_default_users_and_groups,
    add_group_permissions,
)


class Invite(BaseModel, frozen=True):
    user_first_name: str
    user_last_name: str
    user_email: str
    invite_type: str
    action_url: str | None
    resource_id: int | None


class TestInviteView(AuthAPITestCase):

    def setUp(self):
        self.url = "/api/members/"
        setup_default_users_and_groups()

        add_group_permissions(
            "members",
            Member,
            ["view_member"],
        )

        add_group_permissions(
            "admins",
            Member,
            [
                "view_member",
                "change_member",
                "add_member",
                "delete_member",
                "invite_member",
            ],
        )

    def test_request_invite(self):

        invite = Invite(
            user_first_name="User First Name",
            user_last_name="User Last Name",
            user_email="username@domain.com",
            invite_type="basic",
            action_url="post/registration/action",
            resource_id=-1,
        )

        self.authenticate("admin_user")
        response = self.client.post("/accounts/invite", invite.model_dump())
        self.assertEqual(response.status_code, 201, response.data)

        member = MemberDetail(**response.data)
        self.assertEqual(member.username, "username", response.data)
        self.assertEqual(member.verified, False, response.data)
        self.deauthenticate()

        self.assertEqual(len(mail.outbox), 1)
        action_prefix = "http://testserver/accounts/login"
        self.assertTrue(action_prefix in mail.outbox[0].body)



from rest_framework import serializers


class InviteSerializer(serializers.Serializer):

    user_first_name = serializers.CharField(max_length=200)
    user_last_name = serializers.CharField(max_length=200)
    user_email = serializers.CharField(max_length=200)
    invite_type = serializers.CharField(max_length=200)
    action_url = serializers.CharField(max_length=200, allow_null=True)
    resource_id = serializers.IntegerField(allow_null=True)



# Generated by Django 5.2.3 on 2026-02-06 12:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        (
            "ichec_django_core",
            "0011_remove_memberpreferences_member_member_preferences",
        ),
    ]

    operations = [
        migrations.AlterField(
            model_name="formfield",
            name="label",
            field=models.CharField(max_length=400),
        ),
        migrations.AlterField(
            model_name="formfield",
            name="options",
            field=models.CharField(max_length=1000, null=True),
        ),
        migrations.AlterField(
            model_name="formgroup",
            name="label",
            field=models.CharField(max_length=400, null=True),
        ),
    ]



from django.contrib.auth.models import Group

from rest_framework import serializers

from ..models import Member


class GroupListSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        model = Group
        fields = ["url", "name", "id"]
        read_only_fields = ("url", "name", "id")


class GroupDetailSerializer(GroupListSerializer):
    user_set = serializers.HyperlinkedRelatedField(
        many=True, view_name="member-detail", queryset=Member.objects.all()
    )

    class Meta:
        model = Group
        fields = GroupListSerializer.Meta.fields + ["user_set"]
        read_only_fiels = GroupListSerializer.Meta.read_only_fields



from ichec_django_core.models import Feedback

from ichec_django_core.client import FeedbackCreate
from ichec_django_core.test import (
    setup_default_users_and_groups,
    setup_default_feedback,
    add_group_permissions,
    AuthAPITestCase,
)


class TestFeedbackView(AuthAPITestCase):

    template = FeedbackCreate(comments="This is some feedback")
    update_template = {"comments": "This is some edited feedback"}

    def setUp(self):
        self.url = "/api/feedback/"

        setup_default_users_and_groups()
        setup_default_feedback(self.get_user("regular_user"))

        add_group_permissions(
            "admins",
            Feedback,
            ["add_feedback", "view_feedback"],
        )

        add_group_permissions(
            "regular_users",
            Feedback,
            ["add_feedback"],
        )

        add_group_permissions(
            "members",
            Feedback,
            ["add_feedback"],
        )

    def test_list_access(self):

        scenarios = (
            ["", 0, 401, "Public can't list feedback"],
            ["regular_user", 0, 403, "Registered can't list feedback"],
            ["member", 0, 403, "Member can't list feedback"],
            ["admin_user", 3, 200, "Admin can list feedback"],
        )

        for user, count, status, msg in scenarios:
            if user:
                response = self.assert_status(self.auth_list(user), status, msg)
            else:
                response = self.assert_status(self.public_list(), status, msg)

            if status < 400:
                self.assertEqual(response.body.count, count, msg)

    def test_detail_access(self):

        self.assert_201(self.auth_create("regular_user", self.template))

        scenarios = (
            ["", 1, 401, "Public can't view detail"],
            ["regular_user", 1, 403, "Regular user can't view detail"],
            ["member", 1, 403, "Member can't view detail"],
            ["admin_user", 1, 200, "Admin can view detail"],
        )

        for user, item, status, msg in scenarios:
            if user:
                self.assert_status(self.auth_detail(user, item), status, msg)
            else:
                self.assert_status(self.detail(item), status, msg)

    def test_create_permissions(self):

        scenarios = (
            ["", 401, "Public can't create, no auth"],
            ["regular_user", 201, "Registered can create"],
            ["member", 201, "Member can create"],
            ["admin_user", 201, "Admin can create"],
        )

        for user, status, msg in scenarios:
            if user:
                self.assert_status(self.auth_create(user, self.template), status, msg)
            else:
                self.assert_status(self.create(self.template), status, msg)



from pathlib import Path

from ichec_django_core.models import Member
from ichec_django_core.client import MemberIdentifierCreate
from ichec_django_core.client.resources import create_members

from ichec_django_core.test import (
    AuthAPITestCase,
    setup_default_users_and_groups,
    add_group_permissions,
)


class TestMemberView(AuthAPITestCase):
    template = create_members(1)[0]

    def setUp(self):
        self.url = "/api/members/"
        setup_default_users_and_groups()

        add_group_permissions(
            "members",
            Member,
            ["view_member"],
        )

        add_group_permissions(
            "admins",
            Member,
            ["view_member", "change_member", "add_member", "delete_member"],
        )

    def test_list_access(self):

        scenarios = (
            ["", 0, 401, "Public can't list members"],
            ["regular_user", 1, 200, "Registered can list self only"],
            ["member", 5, 200, "Member can list all members"],
            ["admin_user", 5, 200, "Admin can list all members"],
        )

        for user, count, status, msg in scenarios:
            if user:
                response = self.assert_status(self.auth_list(user), status, msg)
            else:
                response = self.assert_status(self.public_list(), status, msg)

            if status < 400:
                self.assertEqual(response.body.count, count, msg)

    def test_detail_access(self):

        self.regular_id = Member.objects.get(username="regular_user").id
        self.other_id = Member.objects.get(username="other_user").id

        scenarios = (
            ["", self.regular_id, 401, "Public can't view member detail"],
            ["regular_user", self.other_id, 404, "Regular user can't see other user"],
            ["regular_user", self.regular_id, 200, "Regular user can see self"],
            ["member", self.regular_id, 200, "Member can see other user"],
            ["admin_user", self.regular_id, 200, "Admin can see other user"],
        )

        for user, item, status, msg in scenarios:
            if user:
                response = self.assert_status(self.auth_detail(user, item), status, msg)
                if status == 200:
                    if user == "regular_user":
                        self.assertTrue(response.body.preferences)
                        # self.assertTrue(response.body.permissions)
                    else:
                        self.assertFalse(response.body.preferences)
                        self.assertFalse(response.body.permissions)
            else:
                self.assert_status(self.detail(item), status, msg)

    def test_create_permissions(self):

        scenarios = (
            ["", 401, "Public can't create, no auth"],
            ["regular_user", 403, "Registered can't create, no perm"],
            ["member", 403, "Member can't create, no perm"],
            ["admin_user", 201, "Admin can create"],
        )

        for user, status, msg in scenarios:
            if user:
                self.assert_status(self.auth_create(user, self.template), status, msg)
            else:
                self.assert_status(self.create(self.template), status, msg)

    def test_profile_upload_permissions(self):

        self.regular_id = Member.objects.get(username="regular_user").id
        scenarios = (
            ["", 401, "profile.png", "Public can't upload profile"],
            ["other_user", 403, "profile.png", "Other user can't upload profile"],
            ["member", 403, "profile.png", "Member can't upload profile"],
            ["regular_user", 204, "profile.png", "Can upload own PNG profile"],
            ["regular_user", 204, "profile.jpg", "Can upload own JPG profile"],
            [
                "regular_user",
                400,
                "profile.svg",
                "Can't upload profile with unsupported extension",
            ],
            [
                "regular_user",
                400,
                "bad_profile.png",
                "Can't upload profile with invalid content",
            ],
            ["admin_user", 20, "profile.png", "Admin can upload profile"],
        )

        for user, status, path, msg in scenarios:
            full_path = Path(__file__).parent / "data" / path
            with open(full_path, "rb") as f:
                content = f.read()

            if user:
                self.assert_status(
                    self.auth_put_file(
                        user, self.regular_id, "profile", content, full_path.name
                    ),
                    status,
                    msg,
                )
            else:
                self.assert_status(
                    self.put_file(self.regular_id, "profile", content, full_path.name),
                    status,
                    msg,
                )

    def test_profile_download_permissions(self):

        self.regular_id = Member.objects.get(username="regular_user").id
        scenarios = (
            ["", 401, "Public user can't download profile or thumb"],
            ["regular_user", 200, "Can download own profile and thumb"],
            ["other_user", 404, "Other user can't download profile or thumb"],
            ["member", 200, "Member can download profile and thumb"],
        )
        full_path = Path(__file__).parent / "data/profile.png"
        with open(full_path, "rb") as f:
            content = f.read()
        self.auth_put_file(
            "regular_user", self.regular_id, "profile", content, full_path.name
        )

        for user, status, msg in scenarios:
            if user:
                response = self.assert_status(
                    self.auth_get(user, self.regular_id, "profile"), status, msg
                )
            else:
                self.assert_status(self.get(self.regular_id, "profile"), status, msg)

        if status < 400:
            self.assertTrue(response.body["basename"] == "profile.png")

    def test_update_authenticated(self):
        response = self.auth_create("admin_user", self.template)
        self.assert_201(response)

        user = response.body
        user_update = response.body.model_copy(
            update={
                "identifiers": [
                    response.body.identifiers[0].model_copy(
                        update={"value": "4321-5678-1234-5678"}
                    )
                ]
            }
        )

        response = self.auth_update("admin_user", user.id, user_update)
        self.assert_200(response)

        self.assertTrue(response.body.identifiers[0].value == "4321-5678-1234-5678")

        user_update = response.body.model_copy(
            update={
                "identifiers": [
                    response.body.identifiers[0],
                    MemberIdentifierCreate(id_type="FREEFORM", value="1234"),
                ]
            }
        )

        response = self.auth_update("admin_user", user.id, user_update)
        self.assert_200(response)

        self.assertTrue(len(response.body.identifiers) == 2)
        user_update = response.body.model_copy(update={"identifiers": []})

        response = self.auth_update("admin_user", user.id, user_update)
        self.assert_200(response)

        self.assertTrue(len(response.body.identifiers) == 0)

    def test_delete_permissions(self):

        scenarios = (
            ["", 401, False, "Public can't delete, no auth"],
            ["regular_user", 403, False, "Registered can't delete"],
            ["other_user", 403, True, "Item member can't delete"],
            ["admin_user", 204, False, "Admin can delete"],
        )

        resources = create_members(len(scenarios))
        for idx, (user, status, member, msg) in enumerate(scenarios):
            response = self.auth_create("admin_user", resources[idx])
            if user:
                self.assert_status(
                    self.auth_delete(user, response.body.id), status, msg
                )
            else:
                self.assert_status(self.delete(response.body.id), status, msg)

    def test_self_view(self):

        response = self.assert_status(
            self.auth_raw_get("regular_user", "/api/self/", action="detail"), 200
        )
        self.assertTrue(response.body.username == "regular_user")



from django.contrib.auth.models import User
from django.core.mail import EmailMultiAlternatives
from django.template.loader import render_to_string
from django.conf import settings
from django.utils.http import urlencode

from rest_framework import permissions
from rest_framework.response import Response
from rest_framework.views import APIView
from rest_framework import status

from ..models import Member
from ..serializers import MemberDetailSerializer, InviteSerializer
from .permissions import get_permission


class CanInvitePermission(permissions.BasePermission):

    def has_permission(self, request, view):
        if not request.user or not request.user.is_authenticated:
            return False

        if request.method != "POST":
            return False

        return request.user.has_perm(get_permission(Member, "invite"))


class InviteUserView(APIView):

    permission_classes = [CanInvitePermission]
    serializer_class = InviteSerializer

    def post(self, request, *args, **kwargs):

        serializer = self.serializer_class(data=request.data)
        serializer.is_valid(raise_exception=True)

        email = serializer.validated_data["user_email"]

        if Member.objects.filter(email=email).count() > 0:
            return Response(
                {"error": "User already exists"}, status=status.HTTP_400_BAD_REQUEST
            )

        email_split = email.split("@")
        if len(email_split) != 2:
            return Response(
                {"error": "User email not in expected format"},
                status=status.HTTP_400_BAD_REQUEST,
            )

        user = User.objects.create_user(
            username=email_split[0],
            first_name=serializer.validated_data["user_first_name"],
            last_name=serializer.validated_data["user_last_name"],
            email=email,
        )

        if settings.EMAIL_HOST_USER:
            self.send_email(
                request, user, email, action_url=serializer.validated_data["action_url"]
            )

        # Send Email
        member = Member.objects.get(id=user.id)
        data = MemberDetailSerializer(member, context={"request": request}).data
        return Response(
            data,
            status=status.HTTP_201_CREATED,
            headers={"Location": str(data["url"])},
        )

    def send_email(self, request, user, email, action_url):

        login_url = "oidc/authenticate" if settings.WITH_OIDC else "accounts/login"
        base_url = request.scheme + "://" + request.get_host() + "/" + login_url
        if action_url:
            full_url = base_url + "/?" + urlencode({"next": action_url})
        else:
            full_url = base_url

        context = {
            "action_url": full_url,
        }

        # Email applicant and facility
        emails_dir = "ichec_django_core/emails"

        template = emails_dir + "/invite_user"
        text = render_to_string(template + ".txt", context=context)
        html = render_to_string(template + ".html", context=context)

        msg = EmailMultiAlternatives(
            subject="User Management Portal Invitation",
            body=text,
            from_email=settings.DEFAULT_EMAIL_SENDER,
            to=[email],
            headers={"List-Unsubscribe": f"<mailto:{settings.DEFAULT_EMAIL_UNSUB}"},
        )

        msg.attach_alternative(html, "text/html")
        msg.send()



# Generated by Django 5.2.1 on 2026-02-27 14:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("ichec_django_core", "0014_alter_member_options"),
    ]

    operations = [
        migrations.AddField(
            model_name="formfield",
            name="tooltip",
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name="formfield",
            name="description",
            field=models.TextField(blank=True, null=True),
        ),
    ]



from rest_framework import status
from rest_framework.test import APITestCase

from django.contrib.auth.models import User

from ichec_django_core.test import (
    AuthAPITestCase,
    setup_default_users_and_groups,
)


class TopLevelViews(AuthAPITestCase):
    def setUp(self):

        setup_default_users_and_groups()
        User.objects.create_user("token_user")

    def test_top_level_not_authenticated(self):
        response = self.client.get("/api/", format="json")
        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)

    def test_manifest(self):
        response = self.client.get("/manifest", format="json")
        self.assertEqual(response.status_code, 200)

    def test_api_auth(self):
        self.assert_200(self.auth_raw_get("token_user", "/api/token/"))



import urllib

from django.urls import include, path
from django.test import override_settings
from django.conf import settings
import requests

import ichec_django_core.urls
from ichec_django_core.views.auth import NoRedirectOIDCLogoutView
from ichec_django_core.auth.backends import MemberOIDCAuthenticationBackend
from ichec_django_core.middleware.oidc import RefreshOIDCAccessToken

from ichec_django_core.test import (
    setup_default_users_and_groups,
    add_group_permissions,
    AuthAPITestCase,
)

urlpatterns = [
    path("oidc/logout/", NoRedirectOIDCLogoutView.as_view(), name="oidc_logout"),
    path("oidc/", include("mozilla_django_oidc.urls")),
]


class TestOIDCBackend(MemberOIDCAuthenticationBackend):

    def __init__(self):
        super().__init__()
        self.last_request = None
        self.default_token_response = {
            "id_token": "test_id_token",
            "access_token": "test_access_token",
            "refresh_token": "test_refresh_token",
        }
        self.next_token_reponse = None

    def get_token(self, request):
        self.last_request = request
        if self.next_token_reponse:
            return self.next_token_reponse
        return self.default_token_response

    def verify_token(self, id_token, nonce):
        return {"payload": "content"}

    def get_userinfo(self, access_token, _id_token, _payload):
        return {"email": "oidc_user@email.com", "preferred_username": "oidc_user"}


def get_response(request):
    return None


class MockResponse:

    def __init__(self):
        self.status_code = 400
        self.text = "Bad Request"


class TestAccessTokenRefresh(RefreshOIDCAccessToken):

    def __init__(self, get_response):
        super().__init__(get_response)
        self.timed_out = False

    def is_expired(self, request):
        return True

    def _do_refresh(self, token_url, token_payload):
        if self.timed_out:
            raise requests.exceptions.HTTPError(response=MockResponse())

        return {
            "access_token": "refreshed_access_token",
            "refresh_token": "refreshed_refresh_token",
        }


@override_settings(WITH_OIDC=True)
@override_settings(OIDC_OP_JWKS_ENDPOINT="/jjkws")
@override_settings(OIDC_RP_CLIENT_ID="test_client")
@override_settings(OIDC_RP_CLIENT_SECRET="client_secret")
@override_settings(OIDC_OP_TOKEN_ENDPOINT="/token_endpoint")
@override_settings(OIDC_AUTHENTICATION_CALLBACK_URL="register")
class TestOIDCLoginView(AuthAPITestCase):

    urls = "ichec_django_core.urls"

    def setUp(self):

        ichec_django_core.urls.urlpatterns += urlpatterns
        setup_default_users_and_groups()
        self.auth_backend = TestOIDCBackend()
        self.refresh_middleware = TestAccessTokenRefresh(get_response)

    def test_login_flow(self):
        response = self.client.get("/oidc/authenticate/?next=abc")

        # Verify response
        self.assertEqual(response.status_code, 302)
        queries = response.url.split("?")[1].split("&")

        query_dict = {}
        for q in queries:
            k, v = q.split("=")
            query_dict[k] = v

        self.assertTrue("redirect_uri" in query_dict)
        query_dict["redirect_uri"] = urllib.parse.unquote(
            query_dict["redirect_uri"]
        ).split("testserver")[1]

        # Verify session state
        session = self.client.session
        self.assertTrue(len(session["oidc_states"]) > 0)
        self.assertTrue(query_dict["state"] in session["oidc_states"])

        # Simulate calling the callback url
        # This won't succeed at logging in the user,
        # as it is difficult to inject the
        # auth backend we need, but it will execute most of the
        # relevant code
        callback_response = self.client.get(
            "/oidc/callback/",
            query_params={"code": "abc123", "state": query_dict["state"]},
        )
        self.assertEqual(response.status_code, 302)

        # Now we manually authenticate
        kwargs = {
            "request": callback_response.wsgi_request,
            "nonce": query_dict["nonce"],
            "code_verifier": session["oidc_states"][query_dict["state"]][
                "code_verifier"
            ],
        }
        user = self.auth_backend.authenticate(**kwargs)

        self.assertTrue(user is not None)
        self.assertEqual(self.auth_backend.last_request["client_id"], "test_client")
        self.assertEqual(
            self.auth_backend.last_request["client_secret"], "client_secret"
        )
        self.assertEqual(user.username, "oidc_user")

        # Verify that we've stored a refresh token
        self.assertEqual(
            callback_response.wsgi_request.session["oidc_refresh_token"],
            "test_refresh_token",
        )

        # Pass the request through the refresh middleware
        refresh_response = self.refresh_middleware.process_request(
            callback_response.wsgi_request
        )
        self.assertEqual(refresh_response, None)

        # Verify that we've stored a new refresh token and reset the expiration timer
        self.assertEqual(
            callback_response.wsgi_request.session["oidc_refresh_token"],
            "refreshed_refresh_token",
        )
        self.assertTrue(
            callback_response.wsgi_request.session["oidc_id_token_expiration"] > 0
        )

        # Now simulate that our oidc session has timed out
        self.refresh_middleware.timed_out = True
        refresh_response = self.refresh_middleware.process_request(
            callback_response.wsgi_request
        )
        self.assertEqual(refresh_response.status_code, 302)



import logging

from django.conf import settings
from django.contrib import auth
from django.utils.module_loading import import_string
from django.http import JsonResponse

from mozilla_django_oidc.views import OIDCLogoutView, get_next_url

logger = logging.getLogger(__name__)


class NoRedirectOIDCLogoutView(OIDCLogoutView):

    def post(self, request):

        logout_url = self.redirect_url
        client_id = settings.OIDC_RP_CLIENT_ID
        next_url = get_next_url(request, "next")

        # Not propagating redirect url right now due to issue with keycloak
        # accepting it
        if next_url and False:
            if not logout_url.endswith("/"):
                logout_url += "/"
            logout_url += f"?client_id={client_id}&post_logout_redirect_uri={next_url}"

        if request.user.is_authenticated:
            # Check if a method exists to build the URL to log out the user
            # from the OP.
            logout_from_op = self.get_settings("OIDC_OP_LOGOUT_URL_METHOD", "")
            if logout_from_op:
                logout_url = import_string(logout_from_op)(request)

            # Log out the Django user if they were logged in.
            auth.logout(request)
        return JsonResponse({"url": logout_url})



from django.test import TestCase

from ichec_django_core.serializers.form import FormSerializer, PopulatedFormSerializer


class TestFormSerializer(TestCase):

    def test_deserialize_form(self):

        data = {
            "groups": [
                {
                    "label": "test",
                    "description": "description",
                    "order": 1,
                    "fields": [
                        {
                            "label": "field_label",
                            "key": "field_key",
                            "required": False,
                            "order": 0,
                            "field_type": "BOOLEAN",
                            "description": "Description",
                            "default": "false",
                        }
                    ],
                }
            ]
        }
        serializer = FormSerializer(data=data)
        self.assertTrue(serializer.is_valid())

        instance = serializer.create(serializer.validated_data)
        self.assertEqual(instance.groups.all()[0].fields.all()[0].key, "field_key")

        serialized = FormSerializer(instance).data
        serialized["groups"][0]["fields"][0]["default"] = "true"

        update_serializer = FormSerializer(data=serialized)
        self.assertTrue(update_serializer.is_valid())
        instance = update_serializer.update(instance, update_serializer.validated_data)
        self.assertEqual(instance.groups.all()[0].fields.all()[0].default, "true")

        data = {"values": [{"value": "test", "field": "1"}]}
        serializer = PopulatedFormSerializer(data=data)
        self.assertTrue(serializer.is_valid())

        instance = serializer.create(serializer.validated_data)
        self.assertEqual(instance.values.all()[0].field.id, 1)

        serialized = PopulatedFormSerializer(instance).data
        serialized["values"][0]["value"] = "updated"
        serialized["values"][0]["field"] = 1

        update_serializer = PopulatedFormSerializer(data=serialized)
        self.assertTrue(update_serializer.is_valid(raise_exception=True))
        instance = update_serializer.update(instance, update_serializer.validated_data)
        self.assertEqual(instance.values.all()[0].value, "updated")



import json

from django.contrib.auth.models import Group

from ichec_django_core.models import Member
from ichec_django_core.client.resources import create_groups
from ichec_django_core.test import (
    AuthAPITestCase,
    setup_default_users_and_groups,
    add_group_permissions,
)


class TestGroupView(AuthAPITestCase):
    def setUp(self):
        self.url = "/api/groups/"
        self.template = create_groups(1)[0]
        setup_default_users_and_groups()

        add_group_permissions(
            "admins",
            Group,
            ["view_group", "change_group", "add_group"],
        )

        add_group_permissions(
            "admins",
            Member,
            ["view_member"],
        )

    def test_list_access(self):

        scenarios = (
            ["", 0, 401, "Public can't list groups"],
            ["regular_user", 1, 200, "Registered can list own groups"],
            ["member", 2, 200, "Member can list own groups"],
            ["admin_user", 3, 200, "Admin can list all groups"],
        )

        for user, count, status, msg in scenarios:
            if user:
                response = self.assert_status(self.auth_list(user), status, msg)
            else:
                response = self.assert_status(self.public_list(), status, msg)

            if status < 400:
                self.assertEqual(response.body.count, count, msg)

    def test_detail_access(self):

        regular_id = Group.objects.get(name="regular_users").id

        scenarios = (
            ["", regular_id, 401, "Public can't view group detail"],
            ["regular_user", regular_id, 403, "Regular user can't view group detail"],
            ["member", regular_id, 403, "Member can't view group detail"],
            ["admin_user", regular_id, 200, "Admin can view group detail"],
        )

        for user, item, status, msg in scenarios:
            if user:
                self.assert_status(self.auth_detail(user, item), status, msg)
            else:
                self.assert_status(self.detail(item), status, msg)

    def test_create_not_authenticated(self):
        data = {"name": "My Group"}
        self.assert_401(self.create(self.template))

    def test_create_authenticated_no_permission(self):
        data = {"name": "My Group"}
        self.assert_403(self.auth_create("regular_user", self.template))

    def test_filter_user(self):
        admin_id = Member.objects.get(username="admin_user").id
        response = self.assert_status(
            self.auth_list("admin_user", {"user": admin_id}), 200
        )
        self.assertEqual(response.body.count, 3)

        member_id = Group.objects.get(name="regular_users").id
        response = self.assert_status(
            self.auth_raw_get("admin_user", "/api/members/", {"groups__id": member_id}),
            200,
        )
        self.assertEqual(response.body["count"], 5)



from ichec_django_core.models import Notification

from ichec_django_core.test import (
    setup_default_users_and_groups,
    setup_default_notifications,
    add_group_permissions,
    AuthAPITestCase,
)


class TestNotificationView(AuthAPITestCase):

    update_template = {"comments": "This is some edited feedback"}

    def setUp(self):
        self.url = "/api/notifications/"

        setup_default_users_and_groups()
        setup_default_notifications(self.get_user("regular_user"))

    def test_list_access(self):

        scenarios = (
            ["", 0, 401, "Public can't list notifications"],
            ["regular_user", 3, 200, "Registered can list own notifications"],
            ["member", 0, 200, "Member can't list other notifications"],
        )

        for user, count, status, msg in scenarios:
            if user:
                response = self.assert_status(self.auth_list(user), status, msg)
            else:
                response = self.assert_status(self.public_list(), status, msg)

            if status < 400:
                self.assertEqual(response.body.count, count, msg)

    def test_detail_access(self):

        scenarios = (
            ["", 1, 401, "Public can't view detail"],
            ["regular_user", 1, 200, "Regular user can view own notification"],
            ["member", 1, 404, "Member can't view other's notifications"],
        )

        for user, item, status, msg in scenarios:
            if user:
                self.assert_status(self.auth_detail(user, item), status, msg)
            else:
                self.assert_status(self.detail(item), status, msg)



import logging

logger = logging.getLogger(__name__)


class RequestLoggerMiddleware:

    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        logger.info("Request path", request.path)
        logger.info("Request host", request.get_host())
        logger.info("Request Cookies", request.COOKIES)
        logger.ingo("Request Headers", request.headers)

        response = self.get_response(request)

        logger.info("Response headers", response.headers)
        logger.info("Response cookies", response.cookies)
        logger.info("Response status", response.status_code)
        return response


class SensitivePathFilter(logging.Filter):

    paths = ("GET /oidc/callback",)

    def filter(self, record):
        for path in self.paths:
            if path in record.getMessage():
                return False
        return True



from functools import partial

from django.contrib.auth.models import User
from rest_framework import status
from rest_framework.test import APITestCase
from django_downloadview import DownloadResponse
from pydantic import BaseModel

from ichec_django_core.client import (
    Paginated,
    Identifiable,
    ModelWithFiles,
    get_type,
    MemberDetail,
)
from ichec_django_core.models import Member


class BaseResponse(BaseModel):
    status_code: int
    body: dict | None


class ListResponse(BaseModel):
    status_code: int
    body: Paginated | dict | None


class DetailResponse(BaseModel):
    status_code: int
    body: Identifiable | dict | None


class AuthAPITestCase(APITestCase):

    def authenticate(self, username: str):
        user = User.objects.get(username=username)
        self.client.force_authenticate(user=user)

    def deauthenticate(self):
        self.client.force_authenticate(user=None)

    def authenticated_op(self, username: str, op):
        self.authenticate(username)
        response = op()
        self.deauthenticate()
        return response

    def get_member_detail(self, name: str) -> DetailResponse:
        user = User.objects.get(username=name)
        self.authenticate(name)
        resp = self.client.get(f"/api/members/{user.id}/", format="json")
        if resp.status_code < 400:
            response = DetailResponse(
                status_code=resp.status_code, body=MemberDetail(**resp.data)
            )
        else:
            response = DetailResponse(status_code=resp.status_code, body=resp.data)
        self.deauthenticate()
        return response

    def get_member_url(self, name):
        detail = self.get_member_detail(name)
        return detail.body.url

    def get_user(self, name: str) -> Member:
        return Member.objects.get(username=name)

    def get_type(self, action: str):
        path = self.url.split("api/")[1].split("/")[0]
        return get_type(path, action)

    def public_list(self, queries: dict | None = None) -> ListResponse:

        response = self.client.get(self.url, query_params=queries, format="json")

        item_t = self.get_type("list")
        if response.status_code < 400:
            return ListResponse(
                status_code=response.status_code,
                body=Paginated(
                    count=response.data["count"],
                    next=response.data["next"],
                    previous=response.data["previous"],
                    results=[item_t(**r) for r in response.data["results"]],
                ),
            )
        else:
            return ListResponse(status_code=response.status_code, body=response.data)

    def auth_list(self, username: str, queries: dict | None = None) -> ListResponse:
        return self.authenticated_op(username, partial(self.public_list, queries))

    def raw_get(
        self, url: str, queries: dict | None = None, action: str = ""
    ) -> BaseResponse | DetailResponse:

        response = self.client.get(url, query_params=queries)
        if action == "detail":
            return DetailResponse(
                status_code=response.status_code,
                body=self.get_type(action)(**response.data),
            )
        else:
            return BaseResponse(status_code=response.status_code, body=response.data)

    def auth_raw_get(
        self, username: str, url: str, queries: dict | None = None, action: str = ""
    ) -> BaseResponse:
        return self.authenticated_op(
            username, partial(self.raw_get, url, queries, action)
        )

    def get(self, resource_id: int, field: str) -> BaseResponse:
        response = self.client.get(self.url + f"{resource_id}/{field}/")

        if isinstance(response, DownloadResponse):
            body = {"basename": response.basename}
        else:
            body = response.data

        return BaseResponse(status_code=response.status_code, body=body)

    def auth_get(self, username: str, resource_id: int, field: str) -> BaseResponse:
        return self.authenticated_op(username, partial(self.get, resource_id, field))

    def detail(self, resource_id: int, role: str = "") -> DetailResponse:
        response = self.client.get(self.url + f"{resource_id}/", format="json")

        type_str = "detail"
        if role:
            type_str += f"-{role}"
        item_t = self.get_type(type_str)
        if response.status_code < 400:
            return DetailResponse(
                status_code=response.status_code, body=item_t(**response.data)
            )
        else:
            return DetailResponse(status_code=response.status_code, body=response.data)

    def auth_detail(
        self, username: str, resource_id: int, role: str = ""
    ) -> DetailResponse:
        return self.authenticated_op(username, partial(self.detail, resource_id, role))

    def create(self, data, format="json") -> DetailResponse:

        if isinstance(data, ModelWithFiles):
            model = data.model
            # files: dict = data.files
        else:
            model = data
            # files = {}

        response = self.client.post(self.url, model.model_dump(), format=format)
        item_t = self.get_type("detail")
        if response.status_code < 400:
            return DetailResponse(
                status_code=response.status_code, body=item_t(**response.data)
            )
        else:
            return DetailResponse(status_code=response.status_code, body=response.data)

    def auth_create(self, username: str, data, format="json") -> DetailResponse:
        return self.authenticated_op(username, partial(self.create, data, format))

    def update(self, resource_id: int, data, format="json") -> DetailResponse:

        if isinstance(data, ModelWithFiles):
            model = data.model
            # files: dict = data.files
        else:
            model = data
            # files = {}
        response = self.client.put(
            self.url + f"{resource_id}/", model.model_dump(), format=format
        )
        item_t = self.get_type("detail")
        if response.status_code < 400:
            return DetailResponse(
                status_code=response.status_code, body=item_t(**response.data)
            )
        else:
            return DetailResponse(status_code=response.status_code, body=response.data)

    def auth_update(self, username: str, resource_id: int, data) -> DetailResponse:
        return self.authenticated_op(username, partial(self.update, resource_id, data))

    def delete(self, resource_id: int) -> BaseResponse:
        response = self.client.delete(self.url + f"{resource_id}/")
        return BaseResponse(status_code=response.status_code, body=None)

    def auth_delete(self, username: str, resource_id: int) -> BaseResponse:
        return self.authenticated_op(username, partial(self.delete, resource_id))

    def put_file(self, resource_id: int, field: str, data, filename) -> BaseResponse:
        response = self.client.put(
            f"{self.url}{resource_id}/{field}/upload",
            data,
            content_type="application/octet-stream",
            headers={"Content-Disposition": f'attachment; filename="{filename}"'},
        )
        return BaseResponse(status_code=response.status_code, body=response.data)

    def auth_put_file(
        self, username: str, resource_id: int, field: str, data, filename: str
    ) -> BaseResponse:
        return self.authenticated_op(
            username, partial(self.put_file, resource_id, field, data, filename)
        )

    def assert_status(self, response, code, msg=""):
        match code:
            case 200:
                return self.assert_200(response, msg)
            case 201:
                return self.assert_201(response, msg)
            case 204:
                self.assert_204(response, msg)
            case 400:
                self.assert_400(response, msg)
            case 401:
                self.assert_401(response, msg)
            case 403:
                self.assert_403(response, msg)
            case 404:
                self.assert_404(response, msg)

    def assert_200(self, response, msg=""):
        self.assertEqual(
            response.status_code,
            status.HTTP_200_OK,
            msg + " | Response: " + str(response.body),
        )
        return response

    def assert_201(self, response, msg=""):
        self.assertEqual(
            response.status_code,
            status.HTTP_201_CREATED,
            msg + " | Response: " + str(response.body),
        )
        return response

    def assert_204(self, response, msg=""):
        self.assertEqual(
            response.status_code,
            status.HTTP_204_NO_CONTENT,
            msg + " | Response: " + str(response.body),
        )

    def assert_401(self, response, msg=""):
        self.assertEqual(
            response.status_code,
            status.HTTP_401_UNAUTHORIZED,
            msg + " | Response: " + str(response.body),
        )

    def assert_403(self, response, msg=""):
        self.assertEqual(
            response.status_code,
            status.HTTP_403_FORBIDDEN,
            msg + " | Response: " + str(response.body),
        )

    def assert_400(self, response, msg=""):
        self.assertEqual(
            response.status_code,
            status.HTTP_400_BAD_REQUEST,
            msg + " | Response: " + str(response.body),
        )

    def assert_404(self, response, msg=""):
        self.assertEqual(
            response.status_code,
            status.HTTP_404_NOT_FOUND,
            msg + " | Response: " + str(response.body),
        )



# Generated by Django 5.2.1 on 2026-02-20 17:53

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        (
            "ichec_django_core",
            "0012_alter_formfield_label_alter_formfield_options_and_more",
        ),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="member",
            options={
                "permissions": [("invite_member", "Can invite members")],
                "verbose_name": "Portal Member",
                "verbose_name_plural": "Portal Members",
            },
        ),
    ]
